//+---------------------------------------------------------------------------+
//|                                                           MASh_Market.mqh |
//|                                         Copyright 2017, Terentyev Aleksey |
//|                                 https://www.mql5.com/ru/users/terentyev23 |
//+---------------------------------------------------------------------------+
#property copyright     "Copyright 2017, Terentyev Aleksey"
#property link          "https://www.mql5.com/ru/users/terentyev23"
#property strict

//+---------------------------------------------------------------------------+
//|   I N C L U D E S                                                         |
//+---------------------------------------------------------------------------+
#include                <Arrays/List.mqh>
#include                "MASh_UI-Class.mqh"


//+---------------------------------------------------------------------------+
//|   D E F I N E S                                                           |
//+---------------------------------------------------------------------------+
// #define             TEST
// For test add "_test" postfix
#ifdef TEST
    #define         PSTFX   "_test"
#else
    #define         PSTFX   ""
#endif

enum SymbolType {
    FOREX,
    CFD,
    FUTURES
};

enum OrderType {
    VIRTUAL = 0,
    REAL
};

class COrder : public CObject
{
public:
    int         ticket;
    string      symbol;
    int         type;
    double      volume;
    double      openPrice;
    datetime    openTime;
    double      stopLoss;
    double      takeProfit;
    int         slippage;
    string      comment;
    int         magicNumber;
    datetime    expiration;
    double      closePrice;
    datetime    closeTime;
    void COrder() : CObject(), ticket(0),
                    symbol(""), type(0),
                    volume(0.0), openPrice(0.0),
                    slippage(0), stopLoss(0.0),
                    takeProfit(0.0), comment(""),
                    magicNumber(0), expiration(0),
                    closePrice(0.0), closeTime(0)
    { }
    void COrder(const string symb, const int cmd,
                const double vol, const double prc,
                const int slppg, const double stop,
                const double take, const string comm,
                const int magic, const datetime exp) : CObject(),
                symbol(symb), type(cmd),
                volume(vol), openPrice(prc),
                slippage(slppg), stopLoss(stop),
                takeProfit(take), comment(comm),
                magicNumber(magic), expiration(exp),
                closePrice(0.0), closeTime(0)
    {
        ticket = (int)TimeCurrent() % 1000000;
        if( type == OP_BUY || type == OP_SELL ) {
            openTime = TimeCurrent();
        }
    }
    void Draw() {}
    bool Load(const int handle)
    {
        string bTime, bType, bVol, bPrice, bStop, bTake;
        string bTime2, bPrice3, bComsn, bSwap, bProfit;
        ReadHistoryLine(handle, bTime, bType, bVol, symbol,
                        bPrice, bStop, bTake, bTime2, 
                        bPrice3, bComsn, bSwap, bProfit, comment);
        ticket = (int)StringToTime(bTime) % 1000000;
        if( bType == "Buy") {
            type = OP_BUY;
        } else if( bType == "Buy Limit" ) {
            type = OP_BUYLIMIT;
        } else if( bType == "Buy Stop" ) {
            type = OP_BUYSTOP;
        } else if( bType == "Sell" ) {
            type = OP_SELL;
        } else if( bType == "Sell Limit" ) {
            type = OP_SELLLIMIT;
        } else if( bType == "Sell Stop" ) {
            type = OP_SELLSTOP;
        }
        volume = StringToDouble(bVol);
        openPrice = StringToDouble(bPrice);
        openTime = StringToTime(bTime);
        stopLoss = StringToDouble(bStop);
        takeProfit = StringToDouble(bTake);
        slippage = 3;
        magicNumber = ticket;
        expiration = StringToTime(bTime2);
        closePrice = StringToDouble(bPrice3);
        closeTime = StringToTime(bTime2);
        return true;
    }
    bool Save(const int handle)
    {
        string bTime, bType, bVol, bPrice, bStop, bTake;
        string bTime2, bPrice3, bComsn, bSwap, bProfit;
        double profit = 0.0;
        bTime = TimeToString(openTime);
        if( type == OP_BUY ) {
            bType = "Buy";
            if( closePrice > 0.0 ) {
                profit = Convert(closePrice - openPrice, StringSubstr(Symbol(), 3, 3), AccountCurrency() );
                profit = volume * MarketInfo(Symbol(), MODE_LOTSIZE) * profit - 0.1;
            }
        } else if( type == OP_BUYLIMIT ) {
            bType = "Buy Limit";
        } else if( type == OP_BUYSTOP ) {
            bType = "Buy Stop";
        } else if( type == OP_SELL ) {
            bType = "Sell";
            if( closePrice > 0.0 ) {
                profit = Convert(openPrice - closePrice, StringSubstr(Symbol(), 3, 3), AccountCurrency() );
                profit = volume * MarketInfo(Symbol(), MODE_LOTSIZE) * profit - 0.1;
            }
        } else if( type == OP_SELLLIMIT ) {
            bType = "Sell Limit";
        } else if( type == OP_SELLSTOP ) {
            bType = "Sell Stop";
        }
        bVol = DoubleToString(volume, 2);
        bPrice = DoubleToString(openPrice, Digits+1);
        bStop = DoubleToString(stopLoss, Digits+1);
        bTake = DoubleToString(takeProfit, Digits+1);
        bTime2 = TimeToString(closeTime);
        bPrice3 = DoubleToString(closePrice, Digits+1);
        bComsn = "0.1";
        bSwap = "";
        bProfit = DoubleToString(profit, 2);
        WriteHistoryLine(handle, bTime, bType, bVol, symbol,
                         bPrice, bStop, bTake, bTime2, 
                         bPrice3, bComsn, bSwap, bProfit, comment);
        return true;
    }
};


//+---------------------------------------------------------------------------+
//|   G L O B A L   V A R I A B L E S                                         |
//+---------------------------------------------------------------------------+
OrderType           _ORDERS_TYPE = REAL;
CList               _ORDERS_LIST, _ORDERS_HISTORY;
COrder*             _CURRENT_ORDER;
color               _COLOR_BUY, _COLOR_SELL, _COLOR_LOSS;
const string        BALANCE_UT = "MASv_BalanceUpdateTime" + PSTFX;
const string        BALANCE_FN = "MASv_BalanceFirstNumber" + PSTFX;


//+---------------------------------------------------------------------------+
//|   F U N C T I O N S                                                       |
//+---------------------------------------------------------------------------+
//+
//+---------------------------------------------------------------------------+
//|   S Y S T E M                                                             |
//+---------------------------------------------------------------------------+
bool NewBar(const int tf = PERIOD_CURRENT)
{
    static datetime lastTime = 0;
    if( lastTime != iTime(Symbol(), tf, 0) ) {      
        lastTime = iTime( Symbol(), tf, 0 );
        return true;
    }
    return false;
};

int IndexOfBar(const datetime time_bar, 
               const string symbol = NULL, const int period = PERIOD_CURRENT)
{
    int index = 0;
    while( time_bar < iTime(symbol, period, index) ) {
        index++;
    }
    return index;
};


//+---------------------------------------------------------------------------+
//|   M A R K E T   D A T A                                                   |
//+---------------------------------------------------------------------------+
double Convert(const double value, const string origin, const string target)
{   // Return converted value from origin to target symbol
    if( origin == target || origin == "" || target == "" ) {
        return value;
    }
    const string prefx = StringSubstr( Symbol(), 6 );
    string symbol1 = origin + target + prefx;
    string symbol2 = target + origin + prefx;
    if( MarketInfo( symbol1, MODE_BID ) > 0.0 ) {
        return value * MarketInfo( symbol1, MODE_BID );
    }
    if( MarketInfo( symbol2, MODE_BID ) > 0.0 ) {
        return value / MarketInfo( symbol2, MODE_BID );
    }
    return -1.0;
};

int SymbolsList(const bool selected, string &symbols[])
{
    string symbolsFileName;
    int symbolsNumber, offset;
    if( selected ) 
        symbolsFileName = "symbols.sel";
    else
        symbolsFileName = "symbols.raw";
    int hFile = FileOpenHistory( symbolsFileName, FILE_BIN|FILE_READ );
    if( hFile < 0 ) 
        return -1;
    if( selected ) {
        symbolsNumber = ( (int)FileSize(hFile) - 4 ) / 128;
        offset = 116;
    } else { 
        symbolsNumber = (int)FileSize(hFile) / 1936;
        offset = 1924;
    }
    ArrayResize( symbols, symbolsNumber );
    if( selected )
        FileSeek( hFile, 4, SEEK_SET );
    for( int i = 0; i < symbolsNumber; i++ ) {
        symbols[i] = FileReadString( hFile, 12 );
        FileSeek( hFile, offset, SEEK_CUR );
    }
    FileClose( hFile );
    return symbolsNumber;
};

double GetBalanceFirstNum()
{   // Update and return balance for first number
    if( Month() != TimeMonth( (datetime)GlobalVariableGet(BALANCE_UT) ) ) {
        GlobalVariableSet( BALANCE_UT, (double)GlobalVariableSet(BALANCE_FN, AccountBalance()) );
    }
    return GlobalVariableGet(BALANCE_FN);
};


//+---------------------------------------------------------------------------+
//|   M O N E Y   M A N A G E M E N T                                         |
//+---------------------------------------------------------------------------+
double GetRiskBuyLot(const double openPrice, const double stopPrice)
{
    return Convert( MathAbs(openPrice-stopPrice) * MarketInfo(Symbol(), MODE_LOTSIZE),
                    StringSubstr(Symbol(), 3, 3), AccountCurrency() );
};

double GetLotSize(const double lotRisk, const double balanceRisk)
{
    double lotMin = MarketInfo(Symbol(), MODE_MINLOT);
    if( lotRisk * lotMin >= balanceRisk || lotRisk <= 0 ) {
        return lotMin;
    } else if( lotMin == 1 ) {
        return MathFloor(balanceRisk / lotRisk); // ?
    } else {
        return NormalizeDouble(balanceRisk / lotRisk - lotMin, 2);
    }
    return -1.0;
};


//+---------------------------------------------------------------------------+
//|   M A R K E T   O P E R A T I O N S                                       |
//+---------------------------------------------------------------------------+
void M_OrdersInitialize(const OrderType orderType,
                        const string folder = "",
                        const color colorBuy = clrLimeGreen,
                        const color colorSell = clrOrangeRed)
{
    _ORDERS_TYPE = orderType;
    _COLOR_BUY = colorBuy;
    _COLOR_SELL = colorSell;
    if( _ORDERS_TYPE == VIRTUAL ) {
        // Load orders data
        string path = "";
        if( StringLen(folder) > 0 ) {
            path = StringConcatenate(folder, "/");
        }
        string filePath = StringConcatenate(path, Symbol(), "_orders", ".csv");
        int hFile = FileOpen(filePath, FILE_READ | FILE_CSV, StringGetChar(";", 0) );
        if( hFile != INVALID_HANDLE ) {
            while( !FileIsEnding(hFile) ) {
                _ORDERS_LIST.Add(new COrder());
                _CURRENT_ORDER = _ORDERS_LIST.GetCurrentNode();
                _CURRENT_ORDER.Load(hFile);
            }
        }
        FileClose(hFile);
        filePath = StringConcatenate(path, Symbol(), "_history", ".csv");
        hFile = FileOpen(filePath, FILE_READ | FILE_CSV, StringGetChar(";", 0) );
        if( hFile != INVALID_HANDLE ) {
            while( !FileIsEnding(hFile) ) {
                _ORDERS_HISTORY.Add(new COrder());
                _CURRENT_ORDER = _ORDERS_HISTORY.GetCurrentNode();
                _CURRENT_ORDER.Load(hFile);
            }
        }
        FileClose(hFile);
    }
};

void M_OrdersDeinitialize(const string folder = "")
{
    if( _ORDERS_TYPE == VIRTUAL ) {
        // Save orders data
        string path = "";
        if( StringLen(folder) > 0 ) {
            path = StringConcatenate(folder, "/");
        }
        string filePath = StringConcatenate(path, Symbol(), "_orders", ".csv");
        int hFile = FileOpen(filePath, FILE_WRITE | FILE_CSV, StringGetChar(";", 0) );
        for( int idx = 0; idx < _ORDERS_LIST.Total(); idx++ ) {
            _ORDERS_LIST.GetNodeAtIndex(idx).Save(hFile);
        }
        FileClose(hFile);
        filePath = StringConcatenate(path, Symbol(), "_history", ".csv");
        hFile = FileOpen(filePath, FILE_WRITE | FILE_CSV, StringGetChar(";", 0) );
        for( int idx = 0; idx < _ORDERS_HISTORY.Total(); idx++ ) {
            _ORDERS_HISTORY.GetNodeAtIndex(idx).Save(hFile);
        }
        FileClose(hFile);
        // Clear
        _ORDERS_LIST.Clear();
        _ORDERS_HISTORY.Clear();
        ObjectsDeleteAll(0, "M_Order");
    }
};

void M_OrdersOnTick()
{
    if( _ORDERS_TYPE == VIRTUAL ) {
        for( int idx = 0; idx < _ORDERS_LIST.Total(); idx++ ) {
            M_OrderSelect(idx, SELECT_BY_POS);
            if( _CURRENT_ORDER.symbol != Symbol() ) {
                continue;
            }
            const int ticket = _CURRENT_ORDER.ticket;
            const int type = _CURRENT_ORDER.type;
            if( type == OP_BUY || type == OP_SELL ) {
                const double stop = _CURRENT_ORDER.stopLoss;
                const double take = _CURRENT_ORDER.takeProfit;
                const double vol = _CURRENT_ORDER.volume;
                if( stop > 0.0 ) {
                    if( type == OP_BUY ) {
                        if( stop >= MarketInfo(Symbol(), MODE_BID) ) {
                            M_OrderClose(ticket, vol, MarketInfo(Symbol(), MODE_BID), 3);
                            M_OrderSelect(ticket, SELECT_BY_TICKET);
                            _CURRENT_ORDER.comment += "_[sl]";
                            continue;
                        }
                    } else {
                        if( stop <= MarketInfo(Symbol(), MODE_ASK) ) {
                            M_OrderClose(ticket, vol, MarketInfo(Symbol(), MODE_ASK), 3);
                            M_OrderSelect(ticket, SELECT_BY_TICKET);
                            _CURRENT_ORDER.comment += "_[sl]";
                            continue;
                        }
                    }
                }
                if( take > 0.0 ) {
                    if( type == OP_BUY ) {
                        if( take <= MarketInfo(Symbol(), MODE_BID) ) {
                            M_OrderClose(ticket, vol, MarketInfo(Symbol(), MODE_BID), 3);
                            M_OrderSelect(ticket, SELECT_BY_TICKET);
                            _CURRENT_ORDER.comment += "_[tp]";
                            continue;
                        }
                    } else {
                        if( take >= MarketInfo(Symbol(), MODE_ASK) ) {
                            M_OrderClose(ticket, vol, MarketInfo(Symbol(), MODE_ASK), 3);
                            M_OrderSelect(ticket, SELECT_BY_TICKET);
                            _CURRENT_ORDER.comment += "_[tp]";
                            continue;
                        }
                    }
                }
            }
            if( type != OP_BUY && type != OP_SELL ) { // Limit & Stop
                if( _CURRENT_ORDER.expiration <= TimeCurrent() ) {
                    M_OrderClose(ticket, 0, 0, 0);
                }
                double openPrice = _CURRENT_ORDER.openPrice;
                double slippage = _CURRENT_ORDER.slippage * MarketInfo(Symbol(), MODE_TICKVALUE);
                if( type == OP_BUYLIMIT || type == OP_BUYSTOP ) {
                    double _ask = MarketInfo(Symbol(), MODE_ASK);
                    if( openPrice - slippage <= _ask ) {
                        if( openPrice + slippage >= _ask ) {
                            _CURRENT_ORDER.type = OP_BUY;
                            _CURRENT_ORDER.openPrice = _ask;
                            _CURRENT_ORDER.openTime = TimeCurrent();
                        }
                    }
                }
                if( type == OP_SELLLIMIT || type == OP_SELLSTOP ) {
                    double _bid = MarketInfo(Symbol(), MODE_BID);
                    if( openPrice - slippage <= _bid ) {
                        if( openPrice + slippage >= _bid ) {
                            _CURRENT_ORDER.type = OP_SELL;
                            _CURRENT_ORDER.openPrice = _bid;
                            _CURRENT_ORDER.openTime = TimeCurrent();
                        }
                    }
                }
            }
            {// Draw
                string objName = StringConcatenate("M_Order", ticket, "_");
                datetime openTime = _CURRENT_ORDER.openTime;
                double openPrice = _CURRENT_ORDER.openPrice;
                double sl = _CURRENT_ORDER.stopLoss;
                double tp = _CURRENT_ORDER.takeProfit;
                color clrOpen;
                // GOTO: change colors
                if( type == OP_BUY || type == OP_BUYLIMIT || type == OP_BUYSTOP) {
                    clrOpen = _COLOR_BUY;
                } else {
                    clrOpen = _COLOR_SELL;
                }
                // arrowOpen
                DrawArrow(objName+"open", openTime, openPrice, 2, clrOpen);
                // GOTO:
                // // tp
                // DrawArrow(objName+"tp", openTime, openPrice, 67, _COLOR_BUY);
                // // sl
                // DrawArrow(objName+"sl", openTime, openPrice, 68, _COLOR_SELL);
            }
        }
        for( int idx = 0; idx < _ORDERS_HISTORY.Total(); idx++ ) {
            M_OrderSelect(idx, SELECT_BY_POS, MODE_HISTORY);
            if( _CURRENT_ORDER.symbol != Symbol() ) {
                continue;
            }
            const int ticket = _CURRENT_ORDER.ticket;
            const int type = _CURRENT_ORDER.type;
            {// Draw
                string objName = StringConcatenate("M_Order", ticket, "_");
                datetime openTime = _CURRENT_ORDER.openTime;
                double openPrice = _CURRENT_ORDER.openPrice;
                datetime closeTime = _CURRENT_ORDER.closeTime;
                double closePrice = _CURRENT_ORDER.closePrice;
                double sl = _CURRENT_ORDER.stopLoss;
                double tp = _CURRENT_ORDER.takeProfit;
                color clrOpen;
                // GOTO: change colors closed orders
                if( type == OP_BUY || type == OP_BUYLIMIT || type == OP_BUYSTOP) {
                    clrOpen = _COLOR_BUY;
                } else {
                    clrOpen = _COLOR_SELL;
                }
                // arrowOpen
                DrawArrow(objName+"open", openTime, openPrice, 2, clrOpen);
                // // tp
                // DrawArrow(objName+"tp", openTime, openPrice, 67, clrGreen);
                // // sl
                // DrawArrow(objName+"sl", openTime, openPrice, 68, clrRed);
                // line
                DrawLine(objName+"line", openTime, openPrice, closeTime, closePrice, clrOpen);
                // arrowClose
                DrawArrow(objName+"close", closeTime, closePrice, 3, clrOpen);
            }
        }
    }
};

bool M_OrderClose(int ticket,       /* номер ордера */
                  double lots,      /* количество лотов */
                  double price,     /* цена закрытия */
                  int slippage,     /* максимальное проскальзывание */
                  color arrow_color = CLR_NONE /* цвет */)
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderClose(ticket, lots, price, slippage, arrow_color);
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        double closePrice;
        M_OrderSelect(ticket, SELECT_BY_TICKET);
        if( _CURRENT_ORDER == NULL ) {
            return false;
        }
        if( _CURRENT_ORDER.type != OP_BUY && _CURRENT_ORDER.type != OP_SELL ) {
            _ORDERS_HISTORY.Add(_ORDERS_LIST.DetachCurrent());
            return true;
        }
        if( _CURRENT_ORDER.type == OP_BUY ) {
            closePrice = MarketInfo(_CURRENT_ORDER.symbol, MODE_BID);
        } else {
            closePrice = MarketInfo(_CURRENT_ORDER.symbol, MODE_ASK);
        }
        if( MathAbs(price - closePrice) > (slippage * MarketInfo(Symbol(), MODE_TICKVALUE)) ) {
            return false;
        }
        if( _CURRENT_ORDER.volume <= lots ) {
            _CURRENT_ORDER.closePrice = closePrice;
            _CURRENT_ORDER.closeTime = TimeCurrent();
            _ORDERS_HISTORY.Add(_ORDERS_LIST.DetachCurrent());
            return true;
        } else {
            _CURRENT_ORDER.volume -= lots;
            _ORDERS_HISTORY.Add(new COrder(_CURRENT_ORDER.symbol, _CURRENT_ORDER.type,
                                           lots, _CURRENT_ORDER.openPrice,
                                           _CURRENT_ORDER.slippage, _CURRENT_ORDER.stopLoss,
                                           _CURRENT_ORDER.takeProfit, _CURRENT_ORDER.comment,
                                           _CURRENT_ORDER.magicNumber, _CURRENT_ORDER.expiration));
            COrder* temp = _ORDERS_HISTORY.GetCurrentNode();
            // temp.ticket = _CURRENT_ORDER.ticket;
            temp.openTime = _CURRENT_ORDER.openTime;
            temp.closePrice = closePrice;
            temp.closeTime = TimeCurrent();
            return true;
        }
    }
    return false;
};

double M_OrderClosePrice()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderClosePrice();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.closePrice;
    }
    return -1;
};

datetime M_OrderCloseTime()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderCloseTime();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.closeTime;
    }
    return -1;
};

string M_OrderComment()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderComment();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.comment;
    }
    return "";
};

double M_OrderCommission()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderCommission();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return 0.0;
    }
    return -1;
};

datetime M_OrderExpiration()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderExpiration();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.expiration;
    }
    return 0;
};

double M_OrderLots()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderLots();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.volume;
    }
    return -1;
};

bool M_OrderModify(int ticket,          /* номер ордера */
                   double price,        /* цена открытия  */
                   double stoploss,     /* stop loss */
                   double takeprofit,   /* take profit */
                   datetime expiration, /* время истечения ордера */
                   color arrow_color = CLR_NONE /* цвет */)
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderModify(ticket, price, stoploss, takeprofit, expiration, arrow_color);
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        M_OrderSelect(ticket, SELECT_BY_TICKET);
        if( _CURRENT_ORDER.closePrice > 0.0 ) {
            return false;
        }
        if( _CURRENT_ORDER.stopLoss == stoploss ) {
            if( _CURRENT_ORDER.takeProfit == takeprofit ) {
                if( _CURRENT_ORDER.expiration == expiration ) {
                    if( _CURRENT_ORDER.openPrice == price ) {
                        return false;
                    }
                }
            }
        }
        _CURRENT_ORDER.stopLoss = stoploss;
        _CURRENT_ORDER.takeProfit = takeprofit;
        if( _CURRENT_ORDER.type != OP_BUY && _CURRENT_ORDER.type != OP_SELL ) {
            _CURRENT_ORDER.openPrice = price;
            _CURRENT_ORDER.expiration = expiration;
        }
    }
    return false;
};

double M_OrderOpenPrice()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderOpenPrice();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.openPrice;
    }
    return -1;
};

datetime M_OrderOpenTime()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderOpenTime();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.openTime;
    }
    return -1;
};

double M_OrderProfit()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderProfit();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        if( _CURRENT_ORDER.type != OP_BUY && _CURRENT_ORDER.type != OP_SELL ) {
            return -1;
        }
        double result = 0.0;
        if( _CURRENT_ORDER.closePrice > 0.0 ) {
            result = _CURRENT_ORDER.openPrice - _CURRENT_ORDER.closePrice;
            if( _CURRENT_ORDER.type == OP_BUY ) {
                result *= -1;
            }
        } else {
            if( _CURRENT_ORDER.type == OP_BUY ) {
                result = MarketInfo(Symbol(), MODE_BID) - _CURRENT_ORDER.openPrice;
            } else {
                result = _CURRENT_ORDER.openPrice - MarketInfo(Symbol(), MODE_ASK);
            }
        }
        return result;
    }
    return -1;
};

bool M_OrderSelect(int index,             /* индекс или тикет ордера */
                   int select,            /* флаг способа выбора */
                   int pool = MODE_TRADES /* источник данных для выбора */)
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderSelect(index, select, pool);
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        if( select == SELECT_BY_POS ) {
            if( pool == MODE_TRADES ) {
                _CURRENT_ORDER = _ORDERS_LIST.GetNodeAtIndex(index);
            } else if( pool == MODE_HISTORY ) {
                _CURRENT_ORDER = _ORDERS_HISTORY.GetNodeAtIndex(index);
            }
            if( _CURRENT_ORDER != NULL ) {
                return true;
            }
        } else if( select == SELECT_BY_TICKET ) {
            for( int idx = 0; idx < _ORDERS_LIST.Total(); idx++ ) {
                _CURRENT_ORDER = _ORDERS_LIST.GetNodeAtIndex(idx);
                if( index == _CURRENT_ORDER.ticket ) {
                    // _CURRENT_ORDER = _ORDERS_LIST.GetCurrentNode();
                    return true;
                }
            }
            for( int idx = 0; idx < _ORDERS_HISTORY.Total(); idx++ ) {
                _CURRENT_ORDER = _ORDERS_HISTORY.GetNodeAtIndex(idx);
                if( index == _CURRENT_ORDER.ticket ) {
                    // _CURRENT_ORDER = _ORDERS_HISTORY.GetCurrentNode();
                    return true;
                }
            }
        }
    }
    return false;
};

int M_OrderSend(const string symbol, const int cmd,
                const double volume, const double price,
                const int slippage,
                const double stoploss, const double takeprofit,
                const string comment = NULL, const int magic = 0,
                const datetime expiration = 0, const color arrow_color = clrNONE)
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderSend(symbol, cmd, volume, price, slippage, stoploss, takeprofit,
                         comment, magic, expiration, arrow_color);
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        _ORDERS_LIST.Add(new COrder(symbol, cmd, volume, price, slippage, stoploss, 
                                    takeprofit, comment, magic, expiration));
        _CURRENT_ORDER = _ORDERS_LIST.GetCurrentNode();
        return _CURRENT_ORDER.ticket;
    }
    return -1;
};

int M_OrdersHistoryTotal()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrdersHistoryTotal();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _ORDERS_HISTORY.Total();
    }
    return -1;
};

double M_OrderStopLoss()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderStopLoss();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.stopLoss;
    }
    return -1;
};

int M_OrdersTotal()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrdersTotal();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _ORDERS_LIST.Total();
    }
    return -1;
};

double M_OrderSwap()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderSwap();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return 0.0;
    }
    return -1;
};

string M_OrderSymbol()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderSymbol();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.symbol;
    }
    return "";
};

double M_OrderTakeProfit()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderTakeProfit();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.takeProfit;
    }
    return -1;
};

int M_OrderTicket()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderTicket();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.ticket;
    }
    return -1;
};

int M_OrderType()
{
    if( _ORDERS_TYPE == REAL ) {
        return OrderType();
    } else if( _ORDERS_TYPE == VIRTUAL ) {
        return _CURRENT_ORDER.type;
    }
    return -1;
};


//+---------------------------------------------------------------------------+
//| Periods                                                                   |
//+---------------------------------------------------------------------------+
ENUM_TIMEFRAMES PeriodMore(const ENUM_TIMEFRAMES period)
{
    int tmp = period == 0 ? Period() : period;
    switch( tmp ) {
        case PERIOD_M1:  return(PERIOD_M5);
        case PERIOD_M5:  return(PERIOD_M15);
        case PERIOD_M15: return(PERIOD_M30);
        case PERIOD_M30: return(PERIOD_H1);
        case PERIOD_H1:  return(PERIOD_H4);
        case PERIOD_H4:  return(PERIOD_D1);
        case PERIOD_D1:  return(PERIOD_W1);
        case PERIOD_W1:  return(PERIOD_MN1);
        case PERIOD_MN1: return(PERIOD_MN1);
        default:         return(PERIOD_CURRENT);
    }
};

ENUM_TIMEFRAMES PeriodMoreX5(const ENUM_TIMEFRAMES period)
{
    int tmp = period == 0 ? Period() : period;
    switch( tmp ) {
        case PERIOD_M1:  return(PERIOD_M5);
        case PERIOD_M5:  return(PERIOD_M30);
        case PERIOD_M15: return(PERIOD_H1);
        case PERIOD_M30: return(PERIOD_H4);
        case PERIOD_H1:  return(PERIOD_H4);
        case PERIOD_H4:  return(PERIOD_D1);
        case PERIOD_D1:  return(PERIOD_W1);
        case PERIOD_W1:  return(PERIOD_MN1);
        case PERIOD_MN1: return(PERIOD_MN1);
        default:         return(PERIOD_CURRENT);
    }
};

ENUM_TIMEFRAMES PeriodLess(const ENUM_TIMEFRAMES period)
{
    int tmp = period == 0 ? Period() : period;
    switch( tmp ) {
        case PERIOD_M1:  return(PERIOD_M1);
        case PERIOD_M5:  return(PERIOD_M1);
        case PERIOD_M15: return(PERIOD_M5);
        case PERIOD_M30: return(PERIOD_M15);
        case PERIOD_H1:  return(PERIOD_M30);
        case PERIOD_H4:  return(PERIOD_H1);
        case PERIOD_D1:  return(PERIOD_H4);
        case PERIOD_W1:  return(PERIOD_D1);
        case PERIOD_MN1: return(PERIOD_W1);
        default:         return(PERIOD_CURRENT);
    }
};

ENUM_TIMEFRAMES PeriodLessX5(const ENUM_TIMEFRAMES period)
{
    int tmp = period == 0 ? Period() : period;
    switch( tmp ) {
        case PERIOD_M1:  return(PERIOD_M1);
        case PERIOD_M5:  return(PERIOD_M1);
        case PERIOD_M15: return(PERIOD_M5);
        case PERIOD_M30: return(PERIOD_M5);
        case PERIOD_H1:  return(PERIOD_M15);
        case PERIOD_H4:  return(PERIOD_H1);
        case PERIOD_D1:  return(PERIOD_H4);
        case PERIOD_W1:  return(PERIOD_D1);
        case PERIOD_MN1: return(PERIOD_W1);
        default:         return(PERIOD_CURRENT);
    }
};

/*
//=====================/ isDayTimeFilter /====================================/
//! Фильтрация по времени
bool SessionOpened(ulong start_session, ulong stop_session)
{
	ulong curr      = TimeCurrent() % 86400;
	start_session   = start_session % 86400;
	stop_session    = stop_session % 86400;
	//RAD_DEBUG_ASSERT(0,"CURRENT:"+string(curr)+" START:"+string(start_session)+" STOP:"+string(stop_session));
	return ((curr >= start_session) && (curr < stop_session)) || !((curr>= start_session) ||(curr < stop_session));
};
//=====================/ BarTimeLeft /========================================/
//! Время в секундах до закрытия бара
//! @return Возвращает время до закрытия бара в сек.
datetime BarTimeLeft(const int period = PERIOD_CURRENT, const string symbol = NULL)
{
	return (datetime)(PeriodSeconds(period) - MathMod(MarketInfo(symbol, MODE_TIME), PeriodSeconds(period)));
};
//========================/ CloseOrders /=====================================/
//! Закрывает все открытые ордера в терминале
//! @return Возвращает количество закрытых ордеров
int CloseOrders(//! Маска выбора типов
				EOrderTypeMask mask=omAll,
				//! Магическое число
				int magic=0,
				//! Инструмент
				string symbol=NULL)
{
	int i;
	int close_cnt=0;
	mask|=omPoolTrades;
	for(i=OrdersTotal()-1; i>=0; i--) {
		if(!isOrderFilter(GetTicket(i,omPoolTrades),mask,magic,symbol))
			continue;
		if(CloseOrder(OrderTicket())==false)
			continue;
		close_cnt++;
	}//for
	return close_cnt;
};
//========================/ SymbolCurrencyDescription /=======================/
//! Строковое описание символа
string SymbolCurrencyDescription(string symbol=NULL)
{
	return SymbolInfoString(symbol, SYMBOL_DESCRIPTION);
}
*/
//===================================================
// Вот так можно проверить останутся ли средства на счете если открыть позицию заданным лотом.

// bool LotCheck(double lot, string Smb)
// {
//    double OneLotPrice=MarketInfo(Smb,MODE_MARGINREQUIRED);
//    double Free   =AccountFreeMargin();
   
//    if (lot*OneLotPrice < Free)
//    { 
//        if(lot>=MarketInfo(Smb,MODE_MINLOT) && lot<MarketInfo(Smb,MODE_MAXLOT))
//        {
         
//          return(true);
//        }  
//    }else 
//    {
//          Print("Not enough money : ",Free);
//          return(false);
//    }   
//    return(false);
// }

// ====================================================================================================

// Вот, кажется нашёл решение, очень простая формула получилась. 
// Узнаём плечо по каждому инструменту, в не зависимости от того что написано в AccountLaverage и не зависимо от размера контракта. 
// Работает для метода расчёта форекс:

// MarketInfo(Symbol(),MODE_TICKVALUE) * Bid / MarketInfo(Symbol(),MODE_MARGINREQUIRED) / MarketInfo(Symbol(),MODE_POINT)

//  - �������� ��������, ���� ������� ������ ��� ���(�� �� ����� ������ ��� ���, �� ��� ��� �������� ���� �� ���� �����), �� ��� ���� � ����������� ������ ������� - ������� �� ������� �� ��������. ����� �������� ������ ������ ��� � ���� �� ����������. ��� ����������� � ����� �����?

// =======================================================================================================
